'''
Created on Mar 31, 2014

@author: Justin
'''

import ECS
from pyglet import resource
import EntityCreator

class SaveState(ECS.Component):
    '''
    Used to store what level to load and saves current level to file.
    '''

    def __init__(self, saveNow=False, loadNow=False):
        '''
        saveNow saves immediately
        loadNow loads immediately
        '''
        super(SaveState, self).__init__(type(self).__name__)
        self.level = 1
        self.saveNow = saveNow
        self.loadNow = loadNow
        
class Killable(ECS.Component):
    '''
    Anything that can die is killable. Killables die on contact with
    a Dangerous entity.
    '''

    def __init__(self, deathEntName, onDieGoToSave=False):
        '''
        deathEntName is the name of the entity that will be created on death.
        onDieGoToSave makes the game reload from last save when the entity dies.
        '''
        super(Killable, self).__init__(type(self).__name__)
        self.deathEntName = deathEntName
        self.onDieGoToSave = onDieGoToSave
        self.dieTimer = 0
        self.dead = False
    
class Dangerous(ECS.Component):
    '''
    Things that can kill killables.
    '''

    def __init__(self, currentlyDeadly=True):
        '''
        currentlyDeadly decides if it will kill on contact or not hurt at all.
        '''
        super(Dangerous, self).__init__(type(self).__name__)
        self.currentlyDeadly = currentlyDeadly
        
class LevelTeleporter(ECS.Component):
    '''
    This component allows the level to change on contact with a certain entity.
    '''

    def __init__(self, level, entType, shouldSave):
        '''
        level is what level to change to.
        entType is the entity type that triggers the level change.
        shouldSave decides if when the level changes the new level should be saved.
        '''
        super(LevelTeleporter, self).__init__(type(self).__name__)
        self.level = level
        self.entType = entType
        self.shouldSave = shouldSave
        
class DeathSaveSystem(ECS.System):
    '''
    This system manages everything having to do with saving, level changes and dying.
    '''
    
    def __init__(self, systemManager):
        super(DeathSaveSystem, self).__init__(systemManager, (["SaveState"],("Killable", "Position"), ("LevelTeleporter", "PhysicsInfo")), 0, False)
    
    def writeLevelToFile(self, sav):
        f = resource.file("save.dat", mode="w")
        f.write(str(sav.level) + "\n")
        f.close()
    
    def loadLevelFromFile(self, sav):
        f = resource.file("save.dat", mode="r")
        sav.level = int(f.readline())
        f.close()
        self.systemManager.clearEntities()
        EntityCreator.loadLevel("level" + str(sav.level), self.systemManager)
        
    def killableCollision(self, player, other):
        kill = player.getComponent("Killable")
        if (other.hasComponent("Dangerous")):
            dang = other.getComponent("Dangerous")
            if (dang.currentlyDeadly and not kill.dead):
                pos = player.getComponent("Position")
                kill.dead = True
                self.systemManager.removeEntity(player)
                death = EntityCreator.createEntity(kill.deathEntName, x=pos.position.x, y=pos.position.y)
                death.addComponent(kill)
                death.dead = True
                self.systemManager.addEntity(death)
    
    def levelLoadCollision(self, ent, other):
        tele = ent.getComponent("LevelTeleporter")
        if (other.entType == tele.entType):
            sav = self.systemManager.getEntityByName("Save").getComponent("SaveState")
            if (tele.shouldSave):
                sav.saveNow = True
                sav.loadNow = True
                sav.level = tele.level
            else:
                sav.level = tele.level
                self.systemManager.clearEntities()
                EntityCreator.loadLevel("level" + str(sav.level), self.systemManager)
    
    def onEntityAdded(self, ent):
        if (ent.hasComponent("Killable") and ent.hasComponent("PhysicsInfo")):
            phys = ent.getComponent("PhysicsInfo")
            phys.collideEvent += self.killableCollision
        elif (ent.hasComponent("LevelTeleporter")):
            phys = ent.getComponent("PhysicsInfo")
            phys.collideEvent += self.levelLoadCollision
    
    def update(self, dt):
        for ent in self.entities:
            if (ent.hasComponent("Killable")):
                kill = ent.getComponent("Killable")
                if (kill.dead):
                    kill.dieTimer += dt
                    if (kill.dieTimer > 2):
                        if (kill.onDieGoToSave):
                            self.systemManager.getEntityByName("Save").getComponent("SaveState").loadNow = True
                        self.systemManager.removeEntity(ent)            
            
            if (ent.hasComponent("SaveState")):
                sav = ent.getComponent("SaveState")
                if (sav.saveNow):
                    sav.saveNow = False
                    self.writeLevelToFile(sav)
                if (sav.loadNow):
                    sav.loadNow = False
                    self.loadLevelFromFile(sav)
        