'''
Created on Mar 18, 2014

@author: Justin
'''

from Events import EventHook

class Entity(object):
    '''
    Every object in game is represented as an entity. Entities have components
    which define their behavior.
    '''
    idCount = 0 # Used to keep ensure all entity names are unique

    def __init__(self, name=None, entType=None, persistant=False):
        '''
        Name is the entities unique name that identifies it only and no other entity can have the same name.
        entType is the type of the entity.
        persistant means that the entity will not be destroyed when all entities are cleared
        '''
        
        self.entType = entType
        self.name = name or self.getUniqueName() # Generate new ID if necessary
        self.componentTable = {} # Store components here so they can be looked up by name
        self.entityDestroyEvent = EventHook()
        self.persistant = persistant
        
    def getUniqueName(self):
        Entity.idCount += 1
        return "entity#" + str(self.idCount)
    
    def addComponent(self, component):
        self.componentTable[component.name] = component
        
        return self # Used to chain function invocation
    
    def removeComponent(self, componentName):
        del self.componentTable[componentName]
        
    def getComponent(self, componentName):
        return self.componentTable[componentName]
    
    def hasComponent(self, componentName):
        return componentName in self.componentTable
    
    def replaceComponent(self, component):
        oldComp = self.getComponent(component.name)
        oldComp.componentReplaced.fire(oldComp, component)
        
        self.removeComponent(component.name)
        self.addComponent(component)
    
class Component(object):
    '''
    This is the base class that components inherit from. Components store
    data that systems use to give components functionality.
    '''
    
    def __init__(self, name="ERROR"):
        '''
        name is the type of component.
        '''
        self.name = name
        self.componentReplaced = EventHook()

class System(object):
    '''
    Systems are what actually process entites that have the required components
    and make them do things.
    '''
    
    def __init__(self, systemManager, componentNameList, priority=0, manualUpdate=False):
        '''
        componentNameList is a list of lists of component combinations that are accepted.
        priority sets the order of when each system is run
        manualUpdate means that the system manager will not call update
        '''
        self.priority = priority
        self.requiredComponents = componentNameList
        self.systemManager = systemManager
        self.manualUpdate = manualUpdate
        self.entities = []
        
    def wantEntity(self, ent):
        for compList in self.requiredComponents:
            found = True
            for compName in compList:
                if (not ent.hasComponent(compName)):
                    found = False
                    break
            if (found):
                return True
                    
        return False
    
    def hasEntity(self, ent):
        return ent in self.entities
    
    def addEntity(self, ent):
        self.entities.append(ent)
        self.onEntityAdded(ent)
        
    def removeEntity(self, ent):
        self.entities.remove(ent)
    
    #Called when system is added
    def onAdded(self):
        pass
    
    #Called when system is removed
    def onRemoved(self):
        pass
    
    def update(self, dt):
        pass
    
    def onEntityAdded(self, ent):
        pass
        
class SystemManager(object):
    '''
    The system manager is what links Entities and systems.
    It is what updates everything.
    '''
    
    def __init__(self):
        '''
        Create lists for storing entities and systems.
        '''
        self.entityList = []
        self.entityNameDict = {}
        self.systemList = []
        self.componentBlobDict = {}
        
        self.entitiesToAdd = []
        self.entitiesToRemove = []
        
    def addEntity(self, ent):
        self.entitiesToAdd.append(ent)
        for sys in self.systemList:
            if (sys.wantEntity(ent)):
                sys.addEntity(ent)
                
    def removeEntity(self, ent, clearPersistant=False):
        if (not ent in self.entitiesToRemove):
            self.entitiesToRemove.append(ent)
        for sys in self.systemList:
            if (sys.hasEntity(ent)):
                if (clearPersistant or not ent.persistant):
                    sys.removeEntity(ent)
                
    def clearEntities(self, clearPersistant=False):
        for ent in self.entityList:
            if (clearPersistant or not ent.persistant):
                self.removeEntity(ent)
                
    def getEntityByName(self, entName):
        try:
            return self.entityNameDict[entName]
        except KeyError:
            return None
        
    def addSystem(self, system):
        self.systemList.append(system)
        self.systemList.sort(key=lambda sys: sys.priority)
        system.onAdded()
        
    def removeSystem(self, system):
        self.systemList.remove(system)
        system.onRemoved()
        
    def getEntsOfType(self, entType):
        ents = []
        for ent in self.entityList:
            if (ent.entType == entType):
                ents.append(ent)
        return ents
        
    def update(self, elapsedTime):
        for sys in self.systemList:
            if (not sys.manualUpdate):
                sys.update(elapsedTime)
                
        for ent in self.entitiesToAdd:
            self.entityList.append(ent)
            self.entityNameDict[ent.name] = ent
        
        for ent in self.entitiesToRemove:
            ent.entityDestroyEvent.fire(ent)
            if (ent in self.entityList):
                self.entityList.remove(ent)
                del self.entityNameDict[ent.name]
            
        self.entitiesToAdd = []
        self.entitiesToRemove = []
        