'''
Created on Mar 24, 2014

@author: Justin
'''


import ECS
from Vector import Vec2d
from pyglet import graphics, resource, sprite, image, text

BACKGROUND_LAYER = 0
MIDDLE_LAYER = 1
FOREGROUND_LAYER = 2

class Position(ECS.Component):
    '''
    This component is used to determine the entities current position.
    It also stores the old position which is used in interpolating between
    physics states.
    '''

    def __init__(self, x, y):
        '''
        x, y is the position.
        '''
        super(Position, self).__init__(type(self).__name__)
        self.position = Vec2d(x, y)
        self.oldPosition = self.position

class Renderable(ECS.Component):
    '''
    Everything that has a picture or animation should be renderable.
    '''

    def __init__(self, imageName="", frames=0, framePeriod=0, width=0, height=0, layer=MIDDLE_LAYER):
        '''
        imageName is the name of the picture file.
        frames is how many frames are in the animation
        framePeriod is how often the animation changes frames
        width, height are only necessary for animations
        layer is which layer to draw on
        '''
        super(Renderable, self).__init__(type(self).__name__)
        self.imageName = imageName
        self.frames = frames
        self.framePeriod = framePeriod
        self.sprite = None
        self.layer = layer
        self.width = width
        self.height = height

class RenderText(ECS.Component):
    '''
    Render text is used to draw text.
    '''
    def __init__(self, text="", font="Times New Roman", fontSize=12, color=(255,255,255,255), layer=MIDDLE_LAYER):
        super(RenderText, self).__init__(type(self).__name__)
        self.text = text
        self.font = font
        self.fontSize = fontSize
        self.fontColor = color
        self.oldColor = color
        self.layer = layer
        self.label = None

class GraphicsSystem(ECS.System):
    '''
    Draws all animations, sprites and text. Also interpolated position for smooth
    rendering.
    '''

    def __init__(self, systemManager):
        super(GraphicsSystem, self).__init__(systemManager , [("Position","Renderable"), ("Position", "RenderText")], 0, True)
        self.batch = graphics.Batch()
        self.layers = []
        
    def onAdded(self):
        for i in range(3):
            self.layers += [graphics.OrderedGroup(i)]
    
    def onRemoved(self):
        self.layers = []

    def entRemoved(self, ent):
        if (ent.hasComponent("Renderable")):
            rend = ent.getComponent("Renderable")
            if (not rend.sprite is None):
                rend.sprite.delete()
        elif (ent.hasComponent("RenderText")):
            rend = ent.getComponent("RenderText")
            rend.label.delete()
    
    def renderReplaced(self, oldComp, newComp):
        if (not oldComp.sprite is None):
            oldComp.sprite.delete()
        oldComp.sprite = None
        oldComp.componentReplaced -= self.renderReplaced
        newComp.componentReplaced += self.renderReplaced
        
            
    def onEntityAdded(self, ent):
        ent.entityDestroyEvent += self.entRemoved
        if (ent.hasComponent("Renderable")):
            ent.getComponent("Renderable").componentReplaced += self.renderReplaced
    
    def update(self, alpha=1.0):
        for ent in self.entities:
            pos = ent.getComponent("Position")
            
            if (ent.hasComponent("Renderable")):
                rend = ent.getComponent("Renderable")
                
                if (rend.sprite is None):
                    if (rend.frames == 0):
                        sprimage = resource.image(rend.imageName)
                    else:
                        square = 0
                        i = 1
                        for i in range(5):
                            if (rend.frames > i*i):
                                square = i
                        
                        rawImage = resource.image(rend.imageName)
                        rawSeq = image.ImageGrid(rawImage, square+1, square+1, rend.width, rend.height, 0, 0)
                        sprimage = image.Animation.from_image_sequence(rawSeq[:rend.frames], rend.framePeriod, True)
                    rend.sprite = sprite.Sprite(sprimage, batch=self.batch, group=self.layers[rend.layer])
                    
                blendedx = (alpha * pos.position.x) + ((1 - alpha) * pos.oldPosition.x)  
                blendedy = (alpha * pos.position.y) + ((1 - alpha) * pos.oldPosition.y)
                
                rend.sprite.set_position(round(blendedx, 0), round(blendedy, 0))
            elif (ent.hasComponent("RenderText")):
                rend = ent.getComponent("RenderText")
                if (rend.label is None):
                    rend.label = text.Label(text=rend.text, font_name=rend.font, font_size=rend.fontSize, color=rend.fontColor, x=pos.position.x, y=pos.position.y, group=self.layers[rend.layer], batch=self.batch)
                if (rend.fontColor != rend.oldColor):
                    rend.label.color = rend.fontColor
                rend.oldColor = rend.fontColor
            
        self.batch.draw()
            