'''
Created on Mar 25, 2014

@author: Justin
'''

import ECS
from pyglet.window import key

class KeyMouseState(object):
    '''
    Used to keep track of the keyboard and mouse state when pushed on the event handler stack.
    '''

    def __init__(self):
        '''
        Initializes dictionaries to store keys in
        '''
        self.keyDic = {}
        self.pressedDic = {}
        self.releasedDic = {}
        self.x = 0
        self.y = 0
    
    #Is key held down
    def isDown(self, key):
        return self.keyDic.get(key, False)
    
    #Is the key just pressed this frame
    def isJustPressed(self, key):
        return self.pressedDic.get(key, False)
    
    #Is the key just released this frame
    def isJustReleased(self, key):
        return self.releasedDic.get(key, False)
    
    def getMousePosition(self):
        return self.x, self.y
    
    #Calling this clears the current state
    def nextFrame(self):
        self.pressedDic.clear()
        self.releasedDic.clear()
        
    def on_key_press(self, symbol, modifiers):
        self.keyDic[symbol] = True
        self.pressedDic[symbol] = True
        
    def on_key_release(self, symbol, modifiers):
        self.keyDic[symbol] = False
        self.releasedDic[symbol] = True
        
    def on_mouse_press(self, x, y, button, modifiers):
        self.keyDic[button] = True
        self.pressedDic[button] = True

    def on_mouse_release(self, x, y, button, modifiers):
        self.keyDic[button] = False
    
    def on_mouse_motion(self, x, y, dx, dy):
        self.x = x
        self.y = y
    
class Controllable(ECS.Component):
    '''
    This component allows the entity to be controlled using the keyboard.
    '''
    
    def __init__(self, walkleft, left, walkright, right, allowHorizontal=True, allowVertical=True):
        super(Controllable, self).__init__(type(self).__name__)
        self.allowHorizontal = allowHorizontal
        self.allowVertical = allowVertical
        self.walkleft = walkleft
        self.walkright = walkright
        self.left = left
        self.right = right
        self.canDoubleJump = True

class MenuItem(ECS.Component):
    '''
    This component allows the entity to be selected in a menu.
    '''
    
    def __init__(self, itemNumber, onSelected, lastItem=0):
        '''
        itemNumber is what number this item is in the menu
        onSelecteed is a function that is called when this item is selected
        lastItem is the number of the last item in the menu
        '''
        
        super(MenuItem, self).__init__(type(self).__name__)
        self.itemNumber = itemNumber
        self.onSelected = onSelected
        self.lastItem = lastItem
    
class InputSystem(ECS.System):
    '''
    This system allows entities to be controlled using the keyboard and also
    keeps track of menu seletion.
    '''
    
    def __init__(self, systemManager, keyState):
        '''
        keyState is a KeyMouseState that was pushed on the event stack.
        '''
        super(InputSystem, self).__init__(systemManager, (("PhysicsInfo", "Controllable", "Renderable"),("MenuItem", "RenderText")), 100, False)
        self.keyState = keyState
        self.currentMenuItem = 0
        self.maxMenu = 999
    
    def update(self, dt):
        if (self.keyState.isJustPressed(key.DOWN)):
            self.currentMenuItem += 1
        elif (self.keyState.isJustPressed(key.UP)):
            self.currentMenuItem -= 1
        if (self.currentMenuItem < 0):
            self.currentMenuItem = self.maxMenu
        elif (self.currentMenuItem > self.maxMenu):
            self.currentMenuItem = 0
        
        menuItems = []
        
        for ent in self.entities:
            if (ent.hasComponent("Controllable")):
                phys = ent.getComponent("PhysicsInfo")
                cont = ent.getComponent("Controllable")
                rend = ent.getComponent("Renderable")
                
                if (cont.allowHorizontal):
                    if (self.keyState.isJustReleased(key.LEFT)):
                        phys.velocity.x = 0
                        ent.replaceComponent(cont.left)
                    elif (self.keyState.isJustReleased(key.RIGHT)):
                        phys.velocity.x = 0
                        ent.replaceComponent(cont.right) 
                        
                    if (not self.keyState.isDown(key.LEFT)):
                        phys.velocity.x = 0
                    elif (not self.keyState.isDown(key.RIGHT)):
                        phys.velocity.x = 0  
                        
                    if (self.keyState.isDown(key.LEFT)):
                        phys.velocity.x = -120
                        if (not rend == cont.walkleft):
                            ent.replaceComponent(cont.walkleft)
                    elif (self.keyState.isDown(key.RIGHT)):
                        phys.velocity.x = 120
                        if (not rend == cont.walkright):
                            ent.replaceComponent(cont.walkright)
                            

                        
                if (cont.allowVertical):
                    if (self.keyState.isJustPressed(key.DOWN)):
                        phys.velocity.y = -120
                    elif (self.keyState.isJustPressed(key.UP)):
                        if (phys.onGround):
                            phys.velocity.y = 225
                            cont.canDoubleJump = True
                        elif (cont.canDoubleJump):
                            phys.velocity.y = 225
                            cont.canDoubleJump = False
                
                if (self.keyState.isJustReleased(key.UP)):
                        phys.velocity.y = min([phys.velocity.y, 75])
            elif (ent.hasComponent("MenuItem")):
                menu = ent.getComponent("MenuItem")
                rend = ent.getComponent("RenderText")
                menuItems.append(menu)
                if (menu.lastItem):
                    self.maxMenu = menu.lastItem
                if (menu.itemNumber == self.currentMenuItem):
                    rend.fontColor = (255, 255, 255, 255)
                    if (self.keyState.isJustPressed(key.ENTER)):
                        menu.onSelected(self.systemManager)
                else:
                    rend.fontColor = (0, 0, 0, 255)
                    