'''
Created on Mar 24, 2014

@author: Justin
'''

import ECS
from Events import EventHook
from Vector import Vec2d

class Rectangle(object):
    '''
    Represents a rectangle and is mainly used as a
    bounding box for the physics system. Rectangles can
    tell when they overlap and by how much.
    '''

    def __init__(self, x, y, width, height):
        self.left = x
        self.right = self.left + width
        self.bottom = y
        self.top = self.bottom + height
        self.centerX = self.left + (width / 2)
        self.centerY = self.bottom + (height / 2)
        self.width = width
        self.height = height
    
    def intersects(self, other):
        return (abs(self.centerX - other.centerX) * 2 < (self.width + other.width)) and (abs(self.centerY - other.centerY) * 2 < (self.height + other.height))
    
    def isColliding(self, otherList):
        for other in otherList:
            if (self.intersects(other)):
                return True
        return False
    
    def getCorrection(self, other):
        halfWidthA = self.width / 2
        halfHeightA = self.height / 2
        halfWidthB = other.width / 2
        halfHeightB = other.height / 2
        
        distanceX = self.centerX - other.centerX
        distanceY = self.centerY - other.centerY
        minDistanceX = halfWidthA + halfWidthB
        minDistanceY = halfHeightA + halfHeightB
        
        if (distanceX > 0):
            depthX = minDistanceX - distanceX
        else:
            depthX = -minDistanceX - distanceX
        if (distanceY > 0):
            depthY = minDistanceY - distanceY
        else:
            depthY = -minDistanceY - distanceY
        
        return Vec2d(depthX, depthY)

class PhysicsInfo(ECS.Component):
    '''
    Stores all necessary info regarding the physics state of the entity.
    '''

    def __init__(self, width=0, height=0, static=False, velocity=Vec2d(0,0), acceleration=Vec2d(0,0), resolveCollision=False):
        '''
        Width and height define the bounding box.
        velocity and acceleration are Vec2d.
        resolveCollision decides wether to just report collisions are acutally try and fix them.
        '''
        super(PhysicsInfo, self).__init__(type(self).__name__)
        self.static = static
        self.velocity = velocity
        self.acceleration = acceleration
        self.collideEvent = EventHook()
        self.width = width
        self.height = height
        self.resolveCollision = resolveCollision
        self.boundingBox = None
        self.onGround = True
        
class PhysicsSystem(ECS.System):
    '''
    Responsible for updating position, detecting collisions, and correcting them.
    '''

    def __init__(self, systemManager, gravity=Vec2d(0, 0)):
        '''
        Gravity is an acceleration that is applied to all physics entities.
        '''
        super(PhysicsSystem, self).__init__(systemManager, [("Position","PhysicsInfo")], 100, False)
        self.gravity = gravity
        
    def resolveCollisions(self, ent, resolveY):
        phys = ent.getComponent("PhysicsInfo")
        pos = ent.getComponent("Position")
        corList = []
        rectList = []
        
        for ent2 in self.entities:
            if (ent2 == ent):
                continue
            phys2 = ent2.getComponent("PhysicsInfo")
            pos2 = ent2.getComponent("Position")
            
            if (phys2.static):
                if (phys2.boundingBox is None):
                        phys2.boundingBox = Rectangle(pos2.position.x, pos2.position.y, phys2.width, phys2.height)
            else:
                phys2.boundingBox = Rectangle(pos2.position.x, pos2.position.y, phys2.width, phys2.height)
            
            if (phys.boundingBox.intersects(phys2.boundingBox)):
                phys2.collideEvent.fire(ent2, ent)
                phys.collideEvent.fire(ent, ent2)
                
                if (phys2.static):
                    corList.append(phys.boundingBox.getCorrection(phys2.boundingBox))
                    rectList.append(phys2.boundingBox)
        
        if (len(corList) == 0 or (not phys.resolveCollision)):
            return
        
        minX = 9999
        minY = 9999
        for cor in corList:
            if (abs(cor.x) < abs(minX)):
                minX = cor.x
            if (abs(cor.y) < abs(minY)):
                minY = cor.y
        
        if (resolveY):
            pos.position.y += minY
            phys.velocity.y = 0
            phys.boundingBox = Rectangle(pos.position.x, pos.position.y, phys.width, phys.height)
            if (minY > 0):
                phys.onGround = True
        else:
            pos.position.x += minX
            phys.velocity.x = 0
            phys.boundingBox = Rectangle(pos.position.x, pos.position.y, phys.width, phys.height)
    
    def update(self, dt):
        for ent in self.entities:
            phys = ent.getComponent("PhysicsInfo")
            pos = ent.getComponent("Position")
            
            if phys.static:
                continue
            oldVelocity = phys.velocity
            phys.velocity = phys.velocity + (self.gravity + phys.acceleration) * dt
            
            pos.oldPosition = pos.position
            
            phys.onGround = False
            pos.position.x = pos.position.x + (oldVelocity.x + phys.velocity.x) * 0.5 * dt
            phys.boundingBox = Rectangle(pos.position.x, pos.position.y, phys.width, phys.height)
            self.resolveCollisions(ent, False)
            pos.position.y = pos.position.y + (oldVelocity.y + phys.velocity.y) * 0.5 * dt
            phys.boundingBox = Rectangle(pos.position.x, pos.position.y, phys.width, phys.height)
            self.resolveCollisions(ent, True)
                